
/*!
 * castaneum
 * Copright(c) 2010 goddamnbugs.com
 * MIT Licensed
 */

exports.version = '0.2.2';

var assert = require('assert'),
    sys = require('sys'),
    fs = require('fs'),
    path = require('path'),
    http = require('http'),
    url = require('url'),
    crypto = require('crypto');

var sizzle = require("../../deps/node-sizzle/node-sizzle"),
    jsdom = require("jsdom");

var castaneum = this;

assert.ok(!!sizzle, "sizzle module.");
assert.ok(!!jsdom, "jsdom module.");

String.prototype.trim = function () {
  return this.replace(/^\s*/, "").replace(/\s*$/, "");
}

function getContentType(filename) {
  switch(path.extname(filename)) {
  case ".html" :
    return "text/html";
  default:
    return "text/plain";
  }
}

function generateHashDigest(data) {
  return crypto.createHash('md5').update(data).digest('base64');
}

exports.readRemoteFile = function(method, urlObj, callback) {
  var port = urlObj.port || 80;
  var client = http.createClient(port, urlObj.hostname);
  if (urlObj.query) {
    var req = client.request(method, urlObj.pathname + '?' + urlObj.query, {'host':urlObj.hostname});
  } else {
    var req = client.request(method, urlObj.pathname, {'host':urlObj.hostname});
  }
  req.on('response', function(res) {
    res.setEncoding('binary');
    var chunks = [],
        i = 0;
    res.on('data', function(chunk) {
      chunks[i++] = chunk;
    });
    res.on('end', function() {
      var response = {};
      response['statusCode'] = res.statusCode;
      response.headers = res.headers;
      response.headers['x-content-source'] = url.format(urlObj);
      response.body = chunks.join('');
      callback(undefined, response, response.body);
    });
  });
  req.end();
};

exports.readLocalFile = function(method, filepath, callback) {
  var response = {};
  response.headers = {};
  require('fs').stat(filepath, function(err, stats) {
    response.headers.data = stats.ctime.toUTCString();
    response.headers['content-type'] = getContentType(filepath);
    response.headers['x-content-source'] = filepath;
    if (method == 'GET') {
      response.headers['content-length'] = stats.size;
      fs.readFile(filepath, 'utf8', function(err, data) {
        response.body = data;
        callback(err, response, response.body);
      });
    } else {
      response.body = "";
      callback(err, response, response.body);  
    }
  });
};

exports.readFile = function(method, uri, callback) {
  var urlObj = url.parse(uri);
  if (urlObj.protocol) {
    this.readRemoteFile(method, urlObj, function(error, response, body) {
      if (method == 'GET') {
        response.headers['content-md5'] = generateHashDigest(body);
      }
      callback(error, response, body);
    });
  } else {
    this.readLocalFile(method, uri, function(error, response, body) {
      if (method == 'GET') {
        response.headers['content-md5'] = generateHashDigest(body);
      }
      callback(error, response, body);
    });
  }
};

exports.get = function(uri, callback) {
  this.readFile('GET', uri, function(error, response, body) {
    callback(error, response, body);
  });
};

exports.head = function(uri, callback) {
  this.readFile('HEAD', uri, function(error, response, body) {
    callback(error, response, body);
  });
};

exports.put = function(uri, data, callback) {
  var urlObj = url.parse(uri);
  if (urlObj.protocol) {
    console.log('put to remote not implemented');
  } else {
    fs.writeFile(uri, data, function(error) {
      if (callback) { callback(error); }
    });
  }
}; 

exports.createDocument = function(response, callback) {
  var jsdom = require('jsdom');
  // jsdom *REALLY* wants to get external scripts by default. bad jsdom. bad.
  jsdom.defaultDocumentFeatures = {};
  
  var window = jsdom.jsdom(response.body).createWindow();
  var document = window.document;
  document.location = response.headers['x-content-source'];

  var sizzle = require('../../deps/node-sizzle/node-sizzle').loadSizzle(document);
  callback(document, sizzle);
};

exports.readSpecSync = function(specPath) {
  var data = require('fs').readFileSync(specPath, 'utf8');
  return JSON.parse(data);
};

exports.loadSpecs = function(specPath) {
  var that = this;
  var files = require('fs').readdirSync(specPath);
    files.forEach(function(file) {
      var filePath = require('path').join(specPath, file);
      var spec = that.readSpecSync(filePath);
      var specKey = require('path').basename(file, '.json');
      that.specs[specKey] = spec;
    });
};

exports.loadSizzle = function(data) {
  // node-htmlparser does not like '>>' or '<<'...change them to continue rather than mess with the parser for now
  data = data.replace('<<', '<').replace('>>', '>');
  var window = jsdom.jsdom(data).createWindow();
  return sizzle.loadSizzle(window.document);  
};

